@@ -199,11 +199,25 @@ void install_socket_ops(void)
 DEFINE_OCTAVE_ALLOCATOR (octave_socket);
 DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_socket, "octave_socket", "octave_socket");
 
+
+
 // This macro must start with DEFUN_DLD so that the automatic collection
-// of function helps can take place.
-#define DEFUN_DLD_SOCKET_CONSTANT(name, help ) \
-DEFUNX_DLD ( #name, F ## name, FS ## name, args, nargout, help) \
-{ return octave_value( name ); };
+// of function helps can take place. To get the code working in
+// multiple versions of octave, we have to check the version number.
+#if !defined(MINORVERSION) || !defined(MAJORVERSION)
+# error "please define MAJORVERSION and MINORVERSION to the octave version numbers"
+#endif
+
+#if MAJORVERSION==3 && MINORVERSION<2
+# define DEFUN_DLD_SOCKET_CONSTANT(name, help )				\
+  DEFUNX_DLD ( #name, F ## name, FS ## name, args, nargout, help)	\
+  {    return octave_value( name ); };					
+#else
+# define DEFUN_DLD_SOCKET_CONSTANT(name, help )				\
+  DEFUNX_DLD ( #name, F ## name, G ## name, args, nargout, help)	\
+  {    return octave_value( name ); };					
+#endif
+
 
 // PKG_ADD: autoload ("AF_UNIX", "sockets.oct");
 DEFUN_DLD_SOCKET_CONSTANT(AF_UNIX, "socket constant" );
@@ -238,6 +252,12 @@ DEFUN_DLD_SOCKET_CONSTANT(SOCK_RAW, "socket constant" );
 DEFUN_DLD_SOCKET_CONSTANT(SOCK_RDM, "socket constant" );
 //DEFUN_DLD_SOCKET_CONSTANT(SOCK_PACKET, "socket constant" );
 
+// PKG_ADD: autoload ("MSG_PEEK", "sockets.oct");
+DEFUN_DLD_SOCKET_CONSTANT(MSG_PEEK, "socket constant" );
+// PKG_ADD: autoload ("MSG_DONTWAIT", "sockets.oct");
+DEFUN_DLD_SOCKET_CONSTANT(MSG_DONTWAIT, "socket constant" );
+// PKG_ADD: autoload ("MSG_WAITALL", "sockets.oct");
+DEFUN_DLD_SOCKET_CONSTANT(MSG_WAITALL, "socket constant" );
 
 std::map< int, octave_socket * > socket_map;
 static bool type_loaded = false;
@@ -299,7 +319,17 @@ void octave_socket::remove_sock_fd(void)
 
 // PKG_ADD: autoload ("socket", "sockets.oct");
 // Function to create a socket
-DEFUN_DLD(socket,args,nargout,"socket(int,int,int)\nSee the socket() man pages\n")
+DEFUN_DLD(socket,args,nargout,
+	  "s=socket(domain,type,protocol)\n"
+	  "Creates a socket. Domain is an integer, where the value AF_INET\n"
+	  "can be used to create an IPv4 socket.\n"
+	  "type is an integer describing the socket. When using IP, specifying "
+	  "SOCK_STREAM gives a TCP socket.\n"
+	  "protocol is currently not used and should be 0 if specified.\n"
+	  "\n"
+	  "If no input arguments are given, default values AF_INET and \n"
+	  "SOCK_STREAM are used.\n"
+	  "See the local socket() reference for more details\n")
 {
   int domain    = AF_INET;
   int type      = SOCK_STREAM;
@@ -358,7 +388,16 @@ DEFUN_DLD(socket,args,nargout,"socket(int,int,int)\nSee the socket() man pages\n
 // PKG_ADD: autoload ("connect", "sockets.oct");
 // function to create an outgoing connection
 DEFUN_DLD(connect,args,nargout, \
-	  "connect(octave_socket,struct)\nSee the connect() man pages")
+	  "status=connect(sock,serverinfo)\n"
+	  "Connects the socket given in sock following the information\n"
+	  "given in the struct serverinfo\n"
+	  "serverinfo shall contain the following fields:\n"
+	  " addr - a string with the host name to connect to\n"
+	  " port - the port number to connect to (an integer)\n"
+	  "\n"
+	  "On successful connect, zero is returned in status.\n"
+	  "\n"
+	  "See the connect() man pages for further details.\n")
 {
   int retval = -1;
   struct sockaddr_in serverInfo;
@@ -374,8 +413,35 @@ DEFUN_DLD(connect,args,nargout, \
   const octave_base_value& struct_serverInfo = args(1).get_rep();
   octave_struct& addrInfo = ((octave_struct&)struct_serverInfo);
 
+#if MINORVERSION <= 2
   string addr = addrInfo.map_value().stringfield("addr");
   int port = addrInfo.map_value().intfield("port");
+#else
+  const Cell addr_cell = addrInfo.map_value().getfield ("addr");
+  string addr;
+  if (addr_cell.numel () == 1 && addr_cell (0).is_string ())
+    {
+      addr = addr_cell (0).string_value ();
+    }
+  else
+    {
+      error ("connect: invalid input: no 'addr' field");
+      return octave_value (-1);
+    }
+
+  const Cell port_cell = addrInfo.map_value().getfield ("port");
+  int port;
+  if (port_cell.numel () == 1 && port_cell (0).is_numeric_type ())
+    {
+      port = port_cell (0).int_value ();
+    }
+  else
+    {
+      error ("connect: invalid input: no 'port' field");
+      return octave_value (-1);
+    }
+#endif
+
 
   // Determine the socket on which to operate
   octave_socket* s = NULL;
@@ -425,7 +491,8 @@ DEFUN_DLD(connect,args,nargout, \
 // PKG_ADD: autoload ("disconnect", "sockets.oct");
 // function to disconnect asocket
 DEFUN_DLD(disconnect,args,nargout, \
-	  "disconnect(octave_socket)\nSince we can't call fclose on the fd directly, use this to disconnect")
+	  "disconnect(octave_socket)\n"
+	  "Since we can't call fclose on the fd directly, use this to disconnect")
 {
   // Determine the socket on which to operate
   octave_socket* s = NULL;
@@ -454,7 +521,8 @@ DEFUN_DLD(disconnect,args,nargout, \
 // PKG_ADD: autoload ("gethostbyname", "sockets.oct");
 // function to get a host number from a host name
 DEFUN_DLD(gethostbyname,args,nargout, \
-	  "gethostbyname(string)\nSee the gethostbyname() man pages")
+	  "gethostbyname(string)\n"
+	  "See the gethostbyname() man pages")
 {
   int nargin = args.length ();
   struct hostent*    hostInfo = NULL;
@@ -486,8 +554,9 @@ DEFUN_DLD(gethostbyname,args,nargout, \
 // PKG_ADD: autoload ("send", "sockets.oct");
 // function to send data over a socket
 DEFUN_DLD(send,args,nargout, \
-	  "send(octave_socket,octave_value)\nSee the send() man pages.  This will only allow the" \
-	  " user to send uint8 arrays or strings")
+	  "send(octave_socket,octave_value,flags)\n"
+	  "See the send() man pages.  This will only allow the\n"
+	  "user to send uint8 arrays or strings\n")
 {
   int retval = 0;
   int flags = 0;
@@ -534,7 +603,7 @@ DEFUN_DLD(send,args,nargout, \
     for ( int i = 0 ; i < d1.length() ; i++ )
       buf[i] = (unsigned char)d1(i);
     retval = ::send( s->get_sock_fd(), (const char*)buf, data.byte_size(), 0 );
-    delete buf;
+    delete[] buf;
   }
   else
   {
@@ -548,8 +617,18 @@ DEFUN_DLD(send,args,nargout, \
 // PKG_ADD: autoload ("recv", "sockets.oct");
 // function to receive data over a socket
 DEFUN_DLD(recv,args,nargout, \
-	  "recv(octave_socket,int)\nSee the send() man pages.  This will only allow the" \
-	  " user to receive uint8 arrays or strings")
+	  "[data,count]=recv(sock,len,flags)\n"
+	  "Requests reading len bytes from the socket given in sock.\n"
+	  "The integer flags parameter can be used to modify the behaviour\n"
+	  "of recv.\n"
+	  "\n"
+	  "The read data is returned in an uint8 array data. The number of\n"
+	  "bytes read is returned in count\n"
+	  "\n"
+	  "You can get non-blocking operation by using the flag MSG_DONTWAIT\n"
+	  "which makes the recv() call return immediately. If there is no\n"
+	  "data, -1 is returned.\n"
+	  "See the recv() man pages for further details.\n")
 {
   int retval = 0;
   int flags = 0;
@@ -577,11 +656,16 @@ DEFUN_DLD(recv,args,nargout, \
   }
   else
   {
-    error("connect: expecting a octave_socket or integer");
+    error("recv: expecting a octave_socket or integer");
     return octave_value(-1);
   }
 
   long len = args(1).int_value();
+  if(len<0) {
+    error("recv: negative receive length requested");
+    return octave_value(-1);
+  }
+
   unsigned char* buf = new unsigned char[ len ];
 #ifndef __WIN32__
   retval = ::recv( s->get_sock_fd(), buf, len, flags );
@@ -589,26 +673,39 @@ DEFUN_DLD(recv,args,nargout, \
   retval = ::recv( s->get_sock_fd(), ( char* )buf, len, flags );
 #endif
 
-  Matrix return_buf(1,retval);
+
   octave_value_list return_list;
-  for ( int i = 0 ; i < retval ; i++ )
-    return_buf(0,i) = buf[i];
-
-  octave_value in_buf(return_buf);
-  octave_value out_buf;
-  OCTAVE_TYPE_CONV( in_buf, out_buf, uint8 );
-  return_list(0) = out_buf;
-	//return_list(0) = return_buf;
-  return_list(1) = retval;
+  //always return the status in the second output parameter
+  return_list(1) = retval; 
+  if(retval<0) {
+    //We get -1 if an error occurs,or if there is no data and the
+    //socket is non-blocking. We should return in both cases.
+  } else if (0==retval) {
+    //The peer has shut down.
+  } else {
+    //Normal behaviour.
+    Matrix return_buf(1,retval);
+    
+    for ( int i = 0 ; i < retval ; i++ )
+      return_buf(0,i) = buf[i];
+    
+    octave_value in_buf(return_buf);
+    octave_value out_buf;
+    OCTAVE_TYPE_CONV( in_buf, out_buf, uint8 );
+    return_list(0) = out_buf;
+  }
 
+  delete[] buf;
+  
   return return_list;
 }
 
 // PKG_ADD: autoload ("bind", "sockets.oct");
 // function to bind a socket
 DEFUN_DLD(bind,args,nargout, \
-	  "bind(octave_socket,int)\nSee the bind() man pages.  This will bind a socket to a" \
-	  " specific port")
+	  "bind(octave_socket,int)\n"
+	  "See the bind() man pages.  This will bind a socket to a" \
+	  " specific port\n")
 {
   int retval = 0;
   if ( args.length() < 2 )
@@ -651,7 +748,8 @@ DEFUN_DLD(bind,args,nargout, \
 // PKG_ADD: autoload ("listen", "sockets.oct");
 // function to listen on a socket
 DEFUN_DLD(listen,args,nargout, \
-	  "listen(octave_socket,int)\nSee the listen() man pages")
+	  "listen(octave_socket,int)\n"
+	  "See the listen() man pages\n")
 {
   int retval = 0;
   if ( args.length() < 2 )
@@ -690,7 +788,8 @@ DEFUN_DLD(listen,args,nargout, \
 // PKG_ADD: autoload ("accept", "sockets.oct");
 // function to accept on a listening socket
 DEFUN_DLD(accept,args,nargout, \
-	  "accept(octave_socket)\nSee the accept() man pages")
+	  "accept(octave_socket)\n"
+	  "See the accept() man pages\n")
 {
   int retval = 0;
   struct sockaddr_in clientInfo;
@@ -756,7 +855,7 @@ DEFUN_DLD(accept,args,nargout, \
 // PKG_ADD: autoload ("load_socket_constants", "sockets.oct");
 // function to load socket constants
 DEFUN_DLD(load_socket_constants,args,nargout, \
-	  "Loads various socket constants like AF_INET, SOCK_STREAM, etc.")
+	  "Loads various socket constants like AF_INET, SOCK_STREAM, etc.\n")
 {
   octave_socket temp();
   return octave_value();
